#!/usr/bin/env python3

import json
import re
import sys
import time
from glob import glob
from pydbus import SessionBus

bus = SessionBus()

STATUS_UPDATE_PERIOD_MILLISECONDS = 1000
DESKTOP_NOTIFICATION_DURATION_SECONDS = 15

class NotificationMessage:
    MESSAGE_LENGTH = 117

    def __init__(self, app_name, summary, body):
        self.app_name = app_name
        self.summary = summary
        self.body = body.replace('\n', ' ')

    def to_components(self):
        if self.app_name == 'Slack':
            components = self.slack_message_to_components()

        elif self.app_name == 'Telegram Desktop':
            components = self.telegram_message_to_components()

        else:
            components = self.regular_message_to_components()

        length = sum(len(component['full_text']) for component in components)

        if length <= self.MESSAGE_LENGTH:
            components[-1]['full_text'] += ' ' * (self.MESSAGE_LENGTH - length)

        else:
            components[-1]['full_text'] = components[-1]['full_text'][:-(length - self.MESSAGE_LENGTH)]

        return components

    def regular_message_to_components(self):
        return [
            {
                'full_text': f'[{self.app_name}] ',
                'separator': False,
                'separator_block_width': 0,
            },
            {
                'full_text': self.summary,
                'color': '#00ffff',
                'separator': False,
                'separator_block_width': 0,
            },
            {
                'full_text': f': {self.body}',
            }
        ]

    def telegram_message_to_components(self):
        self.app_name = 'Telegram'
        return self.regular_message_to_components()

    def slack_message_to_components(self):
        sender = None
        channel = None
        body = self.body

        match = re.match(r'^New message from (.*)', self.summary)
        if match:
            sender = match.group(1)

        else:
            match = re.match(r'^New message in (.*)', self.summary)
            if match:
                channel = match.group(1)

            match = re.match(r'^([^:]*): (.*)', self.body)
            if match:
                sender = match.group(1)
                body = match.group(2)

        components = [
            {
                'full_text': '[Slack] ',
                'separator': False,
                'separator_block_width': 0,
            }
        ]

        if channel:
            components.extend([
                {'full_text': channel, 'color': '#00ff00',
                 'separator': False, 'separator_block_width': 0},
                {'full_text': ': ',
                 'separator': False, 'separator_block_width': 0},
            ])

        if sender:
            components.extend([
                {'full_text': sender, 'color': '#00ffff',
                 'separator': False, 'separator_block_width': 0},
                {'full_text': ': ',
                 'separator': False, 'separator_block_width': 0},
            ])

        components.append({'full_text': body})

        return components


class Notify:
    """
    <node>
      <interface name="org.freedesktop.Notifications">
        <method name="GetServerInformation">
          <arg name="return_name" type="s" direction="out"/>
          <arg name="return_vendor" type="s" direction="out"/>
          <arg name="return_version" type="s" direction="out"/>
          <arg name="return_spec_version" type="s" direction="out"/>
        </method>
        <method name="GetCapabilities">
          <arg name="return_caps" type="as" direction="out"/>
        </method>
        <method name="CloseNotification">
          <arg name="id" type="u" direction="in"/>
        </method>
        <method name="Notify">
          <arg name="app_name" type="s" direction="in"/>
          <arg name="id" type="u" direction="in"/>
          <arg name="icon" type="s" direction="in"/>
          <arg name="summary" type="s" direction="in"/>
          <arg name="body" type="s" direction="in"/>
          <arg name="actions" type="as" direction="in"/>
          <arg name="hints" type="a{sv}" direction="in"/>
          <annotation name="org.qtproject.QtDBus.QtTypeName.In6" value="QVariantMap"/>
          <arg name="timeout" type="i" direction="in"/>
          <arg name="return_id" type="u" direction="out"/>
        </method>
      </interface>
    </node>
    """

    def __init__(self):
        self.last_message = None
        self.last_message_time = 0

    def get_message_to_show(self):
        if time.time() - self.last_message_time < DESKTOP_NOTIFICATION_DURATION_SECONDS:
            return self.last_message.to_components()

        return []

    def Notify(self, app_name, id, icon, summary, body, actions, hints, timeout):
        #print("Notify", args)

        self.last_message = NotificationMessage(app_name, summary, body)
        self.last_message_time = time.time()
        emit_components()
        return 42

    def GetServerInformation(self, *args, **kwargs):
        #print("GetServerInformation", args, kwargs)
        return ("elmord-notify", "https://elmord.org", "0.0.1", "1")

    def GetCapabilities(self, *args, **kwargs):
        #print("GetCapabilities", args, kwargs)
        return ["body", "body-hyperlinks", "body-images"]

    def CloseNotification(self, *args, **kwargs):
        self.last_message_time = 0
        emit_components()
        #print("CloseNotification", args, kwargs)

notify = Notify()

try:
    bus.publish("org.freedesktop.Notifications", notify)
except RuntimeError as error:
    print(str(error), file=sys.stderr)

from gi.repository import GLib

loop = GLib.MainLoop()

[battery_directory] = glob('/sys/class/power_supply/BAT*/')

def get_battery_status():
    with open(f'{battery_directory}/capacity') as f:
        percentage = int(f.read().strip())

    with open(f'{battery_directory}/status') as f:
        status = f.read().strip()

    if status == 'Discharging':
        if percentage < 10:
            color = '#ff4040'
        elif percentage < 25:
            color = '#ffff00'
        else:
            color = '#ffffff'
    elif status in ['Charging', 'Full']:
        color = '#b0b0ff'
    else:
        color = '#ffffff'

    return [
        {
            'full_text': f'{percentage}%',
            'color': color,
        },
    ]

def emit_components():
    components = []
    components.extend(notify.get_message_to_show())
    components.extend([{'full_text': time.strftime('%a %b %d')}])
    components.extend([{'full_text': time.strftime("(%H:%M)", time.gmtime())}])
    components.extend([{'full_text': time.strftime('%H:%M')}])
    components.extend(get_battery_status())

    print(json.dumps(components), ",", flush=True)


def tick(*args, **kwargs):
    emit_components()
    GLib.timeout_add(STATUS_UPDATE_PERIOD_MILLISECONDS, tick)

def print_desktop_notification():
    print(last_desktop_notification_content, flush=True)

print('{"version": 1}\n[')

tick()
loop.run()
